package kr.co.vividnext.sodalive.admin.statistics.member

import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.LocalTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter
import java.time.temporal.ChronoUnit

@Service
class AdminMemberStatisticsService(private val repository: AdminMemberStatisticsRepository) {
    fun getStatistics(
        startDateStr: String,
        endDateStr: String,
        pageable: Pageable
    ): GetMemberStatisticsResponse {
        val startDate = LocalDate.parse(startDateStr)
        val endDate = LocalDate.parse(endDateStr)
        val now = LocalDateTime.now()
            .atZone(ZoneId.of("UTC"))
            .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
            .toLocalDate()

        val dateRange = getPagedDateRange(
            startDate = startDate,
            endDate = endDate.coerceAtMost(now),
            page = pageable.pageNumber + 1,
            pageSize = pageable.pageSize
        )

        if (dateRange == null) {
            throw SodaException("잘못된 접근입니다.")
        }

        var startDateTime = startDate.atStartOfDay()
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        var endDateTime = endDate.atTime(LocalTime.MAX)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val totalSignUpCount = repository.getTotalSignUpCount(startDate = startDateTime, endDate = endDateTime)
        val totalSignUpEmailCount = repository.getTotalSignUpEmailCount(
            startDate = startDateTime,
            endDate = endDateTime
        )
        val totalSignUpKakaoCount = repository.getTotalSignUpKakaoCount(
            startDate = startDateTime,
            endDate = endDateTime
        )
        val totalSignUpGoogleCount = repository.getTotalSignUpGoogleCount(
            startDate = startDateTime,
            endDate = endDateTime
        )
        val totalAuthCount = repository.getTotalAuthCount(startDate = startDateTime, endDate = endDateTime)
        val totalSignOutCount = repository.getTotalSignOutCount(startDate = startDateTime, endDate = endDateTime)
        val totalPaymentMemberCount = repository.getPaymentMemberCount(startDate = startDateTime, endDate = endDateTime)

        startDateTime = dateRange.startDate
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        endDateTime = dateRange.endDate
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val signUpCountInRange = repository.getSignUpCountInRange(
            startDate = startDateTime,
            endDate = endDateTime
        ).associateBy({ it.date }, { it.memberCount })

        val signUpEmailCountInRange = repository.getSignUpEmailCountInRange(
            startDate = startDateTime,
            endDate = endDateTime
        ).associateBy({ it.date }, { it.memberCount })

        val signUpKakaoCountInRange = repository.getSignUpKakaoCountInRange(
            startDate = startDateTime,
            endDate = endDateTime
        ).associateBy({ it.date }, { it.memberCount })

        val signUpGoogleCountInRange = repository.getSignUpGoogleCountInRange(
            startDate = startDateTime,
            endDate = endDateTime
        ).associateBy({ it.date }, { it.memberCount })

        val authCountInRange = repository.getAuthCountInRange(
            startDate = startDateTime,
            endDate = endDateTime
        ).associateBy({ it.date }, { it.memberCount })

        val signOutCountInRange = repository.getSignOutCountInRange(
            startDate = startDateTime,
            endDate = endDateTime
        ).associateBy({ it.date }, { it.memberCount })

        val paymentMemberCountInRange = repository.getPaymentMemberCountInRange(
            startDate = startDateTime,
            endDate = endDateTime
        )

        val paymentMemberCountInRangeMap = paymentMemberCountInRange.associateBy({ it.date }, { it.memberCount })

        val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
        val items = generateSequence(dateRange.endDate) { it.minusDays(1) }
            .takeWhile { !it.isBefore(dateRange.startDate) }
            .map {
                val date = it.format(formatter)
                GetMemberStatisticsItem(
                    date = date,
                    authCount = authCountInRange[date] ?: 0,
                    signUpCount = signUpCountInRange[date] ?: 0,
                    signUpEmailCount = signUpEmailCountInRange[date] ?: 0,
                    signUpKakaoCount = signUpKakaoCountInRange[date] ?: 0,
                    signUpGoogleCount = signUpGoogleCountInRange[date] ?: 0,
                    signOutCount = signOutCountInRange[date] ?: 0,
                    paymentMemberCount = paymentMemberCountInRangeMap[date] ?: 0
                )
            }
            .toList()

        return GetMemberStatisticsResponse(
            totalCount = dateRange.totalDays,
            totalAuthCount = totalAuthCount,
            totalSignUpCount = totalSignUpCount,
            totalSignUpEmailCount = totalSignUpEmailCount,
            totalSignUpKakaoCount = totalSignUpKakaoCount,
            totalSignUpGoogleCount = totalSignUpGoogleCount,
            totalSignOutCount = totalSignOutCount,
            totalPaymentMemberCount = totalPaymentMemberCount,
            items = items
        )
    }

    private fun getPagedDateRange(startDate: LocalDate, endDate: LocalDate, page: Int, pageSize: Int): PagedDateRange? {
        val totalDays = ChronoUnit.DAYS.between(startDate, endDate).toInt() + 1
        val totalPages = (totalDays + pageSize - 1) / pageSize // 전체 페이지 개수 계산

        if (page < 1 || page > totalPages) return null // 페이지 범위를 벗어나면 null 반환

        val rangeEnd = endDate.minusDays((page - 1) * pageSize.toLong()).atTime(LocalTime.MAX)
        val rangeStart = endDate.minusDays((page * pageSize - 1).toLong()).coerceAtLeast(startDate).atStartOfDay()

        return PagedDateRange(rangeStart, rangeEnd, totalDays)
    }
}
