package kr.co.vividnext.sodalive.content.category

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class CategoryService(
    private val repository: CategoryRepository,
    private val contentRepository: AudioContentRepository,
    private val blockMemberRepository: BlockMemberRepository,
    private val categoryContentRepository: CategoryContentRepository
) {
    @Transactional
    fun createCategory(request: CreateCategoryRequest, member: Member) {
        validateTitle(title = request.title)
        val category = repository.findByTitleAndMemberId(title = request.title, memberId = member.id!!)
            ?: repository.save(
                Category(title = request.title).apply {
                    this.member = member
                }
            )
        category.isActive = true

        for (contentId in request.contentIdList) {
            val content = contentRepository.findByIdAndActive(contentId = contentId)
                ?: continue

            val categoryContent = categoryContentRepository.findByContentIdAndCategoryId(
                contentId = contentId,
                categoryId = category.id!!
            ) ?: categoryContentRepository.save(
                CategoryContent().apply {
                    this.content = content
                    this.category = category
                }
            )
            categoryContent.isActive = true
        }
    }

    @Transactional
    fun modifyCategory(request: ModifyCategoryRequest, member: Member) {
        val category = repository.findByIdAndMemberId(categoryId = request.categoryId, memberId = member.id!!)
            ?: throw SodaException("잘못된 요청입니다.")

        if (!request.title.isNullOrBlank()) {
            validateTitle(title = request.title)
            category.title = request.title
        }

        for (contentId in request.addContentIdList) {
            val content = contentRepository.findByIdAndActive(contentId = contentId)
                ?: continue

            val categoryContent = categoryContentRepository.findByContentIdAndCategoryId(
                contentId = contentId,
                categoryId = category.id!!
            ) ?: categoryContentRepository.save(
                CategoryContent().apply {
                    this.content = content
                    this.category = category
                }
            )
            categoryContent.isActive = true
        }

        for (contentId in request.removeContentIdList) {
            val categoryContent = categoryContentRepository.findByContentIdAndCategoryId(
                contentId = contentId,
                categoryId = category.id!!
            ) ?: continue
            categoryContent.isActive = false
        }
    }

    @Transactional
    fun deleteCategory(categoryId: Long, member: Member) {
        val category = repository.findByIdAndMemberId(categoryId = categoryId, memberId = member.id!!)
            ?: throw SodaException("잘못된 요청입니다.")
        category.isActive = false

        categoryContentRepository.deleteByCategoryId(categoryId = categoryId)
    }

    @Transactional
    fun updateCategoryOrders(request: UpdateCategoryOrdersRequest, member: Member) {
        for (index in request.ids.indices) {
            val category = repository.findByIdAndMemberId(categoryId = request.ids[index], memberId = member.id!!)

            if (category != null) {
                category.orders = index + 1
            }
        }
    }

    fun getCategoryList(creatorId: Long, memberId: Long): List<GetCategoryListResponse> {
        val isBlocked = blockMemberRepository.isBlocked(blockedMemberId = memberId, memberId = creatorId)
        if (isBlocked) throw SodaException("잘못된 접근입니다.")

        return repository.findByCreatorId(creatorId = creatorId)
    }

    private fun validateTitle(title: String) {
        if (title.length < 2) throw SodaException("카테고리명은 2글자 이상 입력하세요")
    }
}
