package kr.co.vividnext.sodalive.content.playlist

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.order.OrderRepository
import kr.co.vividnext.sodalive.live.roulette.RedisIdGenerator
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class AudioContentPlaylistService(
    private val idGenerator: RedisIdGenerator,
    private val orderRepository: OrderRepository,
    private val audioContentRepository: AudioContentRepository,
    private val redisRepository: AudioContentPlaylistRedisRepository
) {
    fun createPlaylist(request: CreatePlaylistRequest, member: Member) {
        if (request.contentIdAndOrderList.size >= 30) {
            throw SodaException("플레이 리스트에는 최대 30개의 콘텐츠를 저장할 수 있습니다.")
        }

        val playlistCount = redisRepository.findByMemberId(member.id!!).size
        if (playlistCount >= 10) {
            throw SodaException("플레이 리스트는 최대 10개까지 생성할 수 있습니다.")
        }

        val contentIdAndOrderList = validateAndGetContentIdAndOrderList(
            contentIdAndOrderList = request.contentIdAndOrderList,
            member
        )

        val playlist = AudioContentPlaylist(
            id = idGenerator.generateId(SEQUENCE_NAME),
            memberId = member.id!!,
            title = request.title,
            desc = request.desc,
            contentIdAndOrderList = contentIdAndOrderList
        )

        redisRepository.save(playlist)
    }

    private fun validateAndGetContentIdAndOrderList(
        contentIdAndOrderList: List<PlaylistContentIdAndOrder>,
        member: Member
    ): List<PlaylistContentIdAndOrder> {
        validateContent(
            contentIdList = contentIdAndOrderList.map { it.contentId },
            memberId = member.id!!
        )

        val contentIdAndEndDate = orderRepository.findEndDateByContentId(
            contentIdList = contentIdAndOrderList.map { it.contentId },
            memberId = member.id!!
        )

        val contentIdAndEndDateMap = contentIdAndEndDate.associate { it.contentId to it.endDate }

        return contentIdAndOrderList.map {
            it.endDate = contentIdAndEndDateMap[it.contentId]
            it
        }
    }

    private fun validateContent(contentIdList: List<Long>, memberId: Long) {
        if (contentIdList.isEmpty()) {
            throw SodaException("콘텐츠를 1개 이상 추가하세요")
        }

        checkOrderedContent(
            contentIdList = contentIdList,
            memberId = memberId
        )
    }

    private fun checkOrderedContent(contentIdList: List<Long>, memberId: Long) {
        val orderedContentIdList = orderRepository.findOrderedContent(contentIdList, memberId).toSet()
        val orderedContentMap = contentIdList.associateWith { it in orderedContentIdList }
        val notOrderedContentList = orderedContentMap.filterValues { !it }.keys

        if (notOrderedContentList.isNotEmpty()) {
            throw SodaException("대여/소장하지 않은 콘텐츠는 재생목록에 추가할 수 없습니다.")
        }
    }

    fun updatePlaylist(playlistId: Long, request: UpdatePlaylistRequest, member: Member) {
        if (request.contentIdAndOrderList.size >= 30) {
            throw SodaException("플레이 리스트에는 최대 30개의 콘텐츠를 저장할 수 있습니다.")
        }

        val playlist = redisRepository.findByIdOrNull(id = playlistId)
            ?: throw SodaException("잘못된 요청입니다.")

        if (playlist.memberId != member.id) {
            throw SodaException("잘못된 요청입니다.")
        }

        val contentIdAndOrderList = validateAndGetContentIdAndOrderList(
            contentIdAndOrderList = request.contentIdAndOrderList,
            member
        )

        val updatePlaylist = playlist.copy(
            title = request.title ?: playlist.title,
            desc = request.desc ?: playlist.desc,
            contentIdAndOrderList = contentIdAndOrderList
        )

        redisRepository.save(updatePlaylist)
    }

    fun getPlaylists(member: Member): GetPlaylistsResponse {
        val playlists = redisRepository
            .findByMemberId(memberId = member.id!!)
            .sortedByDescending { it.createdAt }

        return GetPlaylistsResponse(
            totalCount = playlists.size,
            items = playlists.map { playlist ->
                val contentIdAndOrderList = playlist.contentIdAndOrderList.filter {
                    it.endDate == null || it.endDate!! > LocalDateTime.now()
                }

                val contentCount = contentIdAndOrderList.size
                val coverImageUrl = if (contentCount > 0) {
                    audioContentRepository.getCoverImageById(id = contentIdAndOrderList[0].contentId)
                        ?: ""
                } else {
                    ""
                }
                GetPlaylistsItem(
                    id = playlist.id,
                    title = playlist.title,
                    desc = playlist.desc ?: "",
                    contentCount = contentCount,
                    coverImageUrl = coverImageUrl
                )
            }
        )
    }

    fun deletePlaylist(playlistId: Long, member: Member) {
        val playlist = redisRepository.findByIdOrNull(id = playlistId)
            ?: throw SodaException("잘못된 요청입니다.")

        if (playlist.memberId != member.id) {
            throw SodaException("잘못된 요청입니다.")
        }

        redisRepository.delete(playlist)
    }

    fun getPlaylistDetail(playlistId: Long, member: Member): GetPlaylistDetailResponse {
        val playlist = redisRepository.findByIdOrNull(id = playlistId)
            ?: throw SodaException("잘못된 요청입니다.")

        if (playlist.memberId != member.id) {
            throw SodaException("잘못된 요청입니다.")
        }

        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy.MM.dd")
        val createdDate = playlist.createdAt
            .atZone(ZoneId.of("UTC"))
            .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
            .format(dateTimeFormatter)

        val contentIdAndOrderList = playlist.contentIdAndOrderList.filter {
            it.endDate == null || it.endDate!! > LocalDateTime.now()
        }

        val contentList = audioContentRepository.fetchContentForPlaylist(
            contentIdList = contentIdAndOrderList.map { it.contentId }
        )

        val orderMap = contentIdAndOrderList.sortedBy { it.order }
            .mapIndexed { index, item -> item.contentId to index }
            .toMap()

        val sortedContentList = contentList.sortedBy { orderMap[it.id] }

        return GetPlaylistDetailResponse(
            playlistId = playlist.id,
            title = playlist.title,
            desc = playlist.desc ?: "",
            createdDate = createdDate,
            contentCount = sortedContentList.size,
            playlistCoverImageList = sortedContentList.take(4).map { it.coverUrl },
            contentList = sortedContentList
        )
    }

    companion object {
        const val SEQUENCE_NAME = "AudioContentPlaylist:sequence"
    }
}
