package kr.co.vividnext.sodalive.notice

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.notice.QServiceNotice.serviceNotice
import org.springframework.data.domain.Pageable
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
interface ServiceServiceNoticeRepository : JpaRepository<ServiceNotice, Long>, ServiceNoticeQueryRepository

interface ServiceNoticeQueryRepository {
    fun getNoticeTotalCount(): Int
    fun getNoticeList(pageable: Pageable): List<NoticeItem>
    fun getLatestNotice(): NoticeItem?
}

@Repository
class ServiceNoticeQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : ServiceNoticeQueryRepository {
    override fun getNoticeTotalCount(): Int {
        return queryFactory
            .select(serviceNotice.id)
            .from(serviceNotice)
            .where(serviceNotice.isActive.isTrue)
            .fetch()
            .size
    }

    override fun getNoticeList(pageable: Pageable): List<NoticeItem> {
        return queryFactory
            .select(
                QNoticeItem(
                    serviceNotice.id,
                    serviceNotice.title,
                    serviceNotice.content,
                    getFormattedDate(serviceNotice.createdAt)
                )
            )
            .from(serviceNotice)
            .where(serviceNotice.isActive.isTrue)
            .offset(pageable.offset)
            .limit(pageable.pageSize.toLong())
            .orderBy(serviceNotice.id.desc())
            .fetch()
    }

    override fun getLatestNotice(): NoticeItem? {
        return queryFactory
            .select(
                QNoticeItem(
                    serviceNotice.id,
                    serviceNotice.title,
                    serviceNotice.content,
                    getFormattedDate(serviceNotice.createdAt)
                )
            )
            .from(serviceNotice)
            .where(serviceNotice.isActive.isTrue)
            .orderBy(serviceNotice.id.desc())
            .fetchFirst()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )
    }
}
